/* SPDX-License-Identifier: LGPL-3.0-or-later */

/*
 * Copyright (C) 2023 Perry Werneck <perry.werneck@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

 /**
  * @brief Brief Describe 'OEMString' value.
  */

 #pragma once

 #ifdef HAVE_CONFIG_H
	#include <config.h>
 #endif // HAVE_CONFIG_H

 /*
 #include <smbios/defs.h>
 #include <smbios/value.h>
 #include <string>
 #include <cstdint>

 namespace SMBios {

 	class SMBIOS_API OEMString : public Abstract::Value {
	private:
		size_t index = 0;
		const char *ptr = nullptr;
		std::string oname;

	public:

		/// @brief Construct an OEM string value.
		/// @param index Index (for string name).
		/// @param str Pointer to the beginning of string.
		OEMString(size_t index, const char *str);

		std::string as_string() const override;
		const char *name() const noexcept override;
		const char *description() const noexcept override;
		bool valid() const override;
		uint64_t as_uint64() const override;
		unsigned int as_uint() const override;
		Abstract::Value & next() override;

 	};

 }
 */

